
#ifndef __HistogramDialogComboBox__
#define __HistogramDialogComboBox__


// ----------------------------------------------------------------------------
// wx headers inclusion.
// For compilers that support precompilation, includes <wx/wx.h>.
// ----------------------------------------------------------------------------

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/wx.h>
#include <vector>

#include "HistogramDialogComboBoxItem.h"
#include <fstream>
#include <wx/bmpcbox.h>
#include "vtkImageData.h"

#ifdef WIN32
#define FILENAME "/data/colorsfunction.txt"
#else
#define FILENAME "~/.creaMaracasVisu/data/colorsfunction.txt"
#endif


class HistogramDialogComboBox : public wxPanel
{
public:
	/*
	 Constructor
	*/
	HistogramDialogComboBox(wxWindow* parent);
	//HistogramDialogComboBox(int bar_width,int bar_height);
	~HistogramDialogComboBox();

	void SetColors(std::vector<double> greyvect, std::vector<double> redvect, std::vector<double> greenvect, std::vector<double> bluevect);
	wxBitmap GetBitmap();

	std::vector<double> getGreyVector(){return _greyvect;}
	std::vector<double> getRedVector(){return _redvect;}
	std::vector<double> getGreenVector(){return _greenvect;}
	std::vector<double> getBlueVector(){return _bluevect;}

	void OnEditBitmapCombo(wxCommandEvent& event);
	void OnBitmapComboItemSelected(wxCommandEvent& event);

	void OnColorLevelChanged(wxCommandEvent& event);	
	void OnWindowLevelChanged(wxCommandEvent& event);	

	void setImageData(vtkImageData* img);

	/**
	**	Returns two vectors, the grey level of the point and its value, the value is between [0,1]
	**/
	void GetValuesPointsFunction(std::vector<double>& greylevel,std::vector<double>& value);

	/**
	**	Returns two vectors, the grey level of the point and its value, the red, green
	**	and blue value is between [0,1]
	**/
	void GetValuesColorPointsFunction(std::vector<double>& greylevel,
									std::vector<double>& red,
									std::vector<double>& green,
									std::vector<double>& blue);

	double GetWindowLevel();
	double GetColorLevel();

private:

	std::vector<double> _greyvect;
	std::vector<double> _redvect;
	std::vector<double> _greenvect;
	std::vector<double> _bluevect;
	std::vector<double> _greyvecttransfer;
	std::vector<double> _value;
	vtkImageData* _img;
	double _maxgreyvalue;
	int _currentitem;	

	/*
	* Represents the color of the backGround. Default color is the parent color. 
	*/		

	wxBitmapComboBox*	_bitmapcombo;
	wxSlider*	_slidercolor;
	wxSlider*	_sliderwindowlevel;
	std::vector<HistogramDialogComboBoxItem*> _bitmapsitems;
	wxBitmap* colorBar_Bitmap;
	wxSizer* _bitmapsizer;

	wxSizer* getBitmapCombo();
	wxSizer* getSlidersWlCo();
	wxBitmapComboBox* getBitmapComboElements();
	void OnLoadComboBoxData(std::vector<HistogramDialogComboBoxItem*>& itembitmaps,std::string filename);
	void saveCurrentConfiguration(std::vector<HistogramDialogComboBoxItem*>& itembitmaps,std::string filename);
	void setSlidersValue();


	
};

#endif



